# Development

## Philosophy

1. First create something that works (to provide business value).
1. Then something that’s beautiful (to lower maintenance costs).
1. Finally works on performance (to avoid wasting time on premature
   optimizations).

## Stability policy

This project more or less follows [Semantic Versioning](https://semver.org/).

Which boils down to the following rules of thumb regarding stability:

- **Patch releases**: `0.x.n` → `0.x.(n+1)` upgrades

  Are bug-fix only. These releases must not break anything and keep
  backward-compatibility with `0.x.*` and `0.(x-1).*` series.

- **Minor releases**: `0.n.*` → `0.(n+1).0` upgrades

  Includes any non-bugfix changes. These releases must be backward-compatible
  with any `0.n.*` version but are allowed to drop compatibility with the
  `0.(n-1).*` series and below.

- **Major releases**: `n.*.*` → `(n+1).0.0` upgrades

  Make no promises about backwards-compability. Any API change requires a new
  major release.

## Build status

{gh}`main branch <tree/main>`:
[![Unittests status](https://github.com/kdeldycke/mail-deduplicate/actions/workflows/tests.yaml/badge.svg?branch=main)](https://github.com/kdeldycke/mail-deduplicate/actions/workflows/tests.yaml?query=branch%3Amain)
[![Coverage status](https://codecov.io/gh/kdeldycke/mail-deduplicate/branch/main/graph/badge.svg)](https://codecov.io/gh/kdeldycke/mail-deduplicate/branch/main)

## Setup environment

This **step is required** for all the other sections from this page.

Check out latest development branch:

```shell-session
$ git clone git@github.com:kdeldycke/mail-deduplicate.git
$ cd ./mail-deduplicate
$ git checkout main
```

Install package in editable mode with all development dependencies:

```shell-session
$ python3 -m pip install poetry
$ poetry install
```

Now you’re ready to hack and abuse git!

## Test `mdedup` development version

After the steps above, you are free to play with the bleeding edge version of `mdedup`:

```shell-session
$ poetry run mdedup --version
mdedup, version 4.13.0-dev
(...)
```

## Unit-tests

Run unit-tests with:

```shell-session
$ poetry run pytest
```

## Coding style

{gh}`Code linting is handled by a workflow <blob/main/.github/workflows/lint.yaml>`.

## Documentation

The documentation you’re currently reading can be built locally with
[Sphinx](https://www.sphinx-doc.org):

```shell-session
$ poetry run sphinx-build -b html ./docs ./docs/html
```

The generation of API documentation is
{gh}`covered by a dedicated workflow <blob/main/.github/workflows/docs.yaml>`.

## Screenshots

Project screenshots found in the documentation and the `readme.md` file needs
to be refreshed by hand once in a while.

To produce clean and fancy terminals screenshots, use either:

- [Graphite Shot](https://graphite-shot.now.sh)
- [Carbon](https://github.com/carbon-app/carbon)
- [CodeKeep](https://codekeep.io/screenshot)

## Changelog

Before a release, the maintainers will review and rewrite the changelog to make
it clean and readable. Inspiration can be drawn from the
[keep a changelog manifesto](https://keepachangelog.com).

Changes can be inspected by using the comparison URL between the last tagged
version and the `main` branch. This link is available at the top of the
{doc}`/changelog`.

## Release process

All steps of the release process and version management are automated in the
{gh}`changelog.yaml <blob/main/.github/workflows/changelog.yaml>` and
{gh}`release.yaml <blob/main/.github/workflows/release.yaml>` and workflows.

All there's left to do is to:

- {gh}`check the open draft prepare-release PR <workflows/pulls?q=is%3Apr+is%3Aopen+head%3Aprepare-release>`
  and its changes,
- click the `Ready for review` button,
- click the `Rebase and merge` button,
- let the workflows tag the release and set back the `main` branch into a
  development state.

## Version bump

Versions are bumped to their next `patch` revision during the release process
above by the
{gh}`release.yaml workflow <blob/main/.github/workflows/release.yaml>`.

At any point during development, you can bump the version by merging either the
`minor-version-increment` or `major-version-increment` branch, each available
in their own PR.
